function [Fmin,Fmax] = circle(a,b,c,d,kmin,kmax)

% CIRCLE  An implementation of the Circle Criterion for nonlinear
%         feedback control system with a linear part G(s) and a
%         nonlinearity lying in a sector [ F_min, F_max ], i.e., 
%                        F_min < f(y,t)/y < F_max
%         The nonlinear system is uniformly asymptotically stable
%         in the large (UASIL) if the circle whose diameter is 
%         defined by the points -1/F_max < Re(G(jw)) < -1/F_min,
%         Im(G(jw)) = 0 contains no interior points of the right-
%         half-plane (RHP) mapping of G(s).
% 
%         CIRCLE(A,B,C,D,Fmin,Fmax) will check if the CC is satisfied
%         for the sector-bounded nonlinearity and G(s) = C(sI-A) B + D
% 
%         CIRCLE(A,B,C,D,Fmin) will determine the maximum value of
%         F_max for which the CC is satisfied.
% 
%         CIRCLE(A,B,C,D,NaN,Fmax) will determine the minimum value of
%         F_min for which the CC is satisfied.
%  
%         [FMIN,FMAX] = CIRCLE(A,B,C,D,FMIN) will return the sector
%         limits F_min & F_max for stability in addition to plotting.
%
%         CIRCLE(NUM,DEN,Fmin,Fmax) et cetera are corresponding forms
%         for G(s) provided in ratio of polynomial form. 
%  
%         This routine is built on NEWNYQ, a new & improved version of
%         the Nyquist criterion with advanced graphics and reporting.
%
%         C. Chan and J.H. Taylor, Univ. of New Brunswick, Feb. 1997.
%
%         Reference: Narendra & Taylor "Frequency Domain Criteria for
%         Absolute Criteria", Academic Press, 1973.

% 96 Sept 14,17,18,19,20,21,22,23,24,25 Oct 21,24,26,30, Nov 1,21,27 
% 97 Jan 15 May 7,15,22 June 3,4 Oct 15
% 2004 July 10
mynargout = nargout;
if nargin == 3 | nargin == 4,
   num = a; den = b;      
   [Grootloc,freq] = findrt(num,den);
elseif nargin == 5 | nargin == 6,
   [Grootloc,freq,num,den] = findrt(a,b,c,d);
else
   warndlg('Wrong number of input arguments'); % works if you run the function from the m-file
end 
% check if it is kmin to be determined 
if nargin == 4,
   if isnan(c),       % here c = kmin
      den = -den; Grootloc = -Grootloc;
   elseif c > d,      % here d = kmax
      error('Fmin must be smaller than Fmax'); 
   end
end
if nargin == 6,
   if isnan(kmin),
      den = -den; Grootloc = -Grootloc; 
   elseif kmin > kmax,
      error('Fmin must be smaller than Fmax');
   end
end  
% to get rid of the warnings (divided by zero) and (log of zero)
% that pop up after running the nyquist 
Gre=[];Gim=[];
if any(isreal(Gre))| any(isreal(Gim))
    %%jht_17_nov_05  [Gre,Gim,w] = nyq4p2(num,den,[0.01:0.01:1000]);
    [Gre,Gim,w] = nyq4p2(num,den);
elseif any(isnan(Gre))|any(isnan(Gim))
       Gre=[];Gim=[];
end
[Grootloc,freq] = addr(num,den,Grootloc,freq); 
%%jht_17_nov_05  Gre,Gim,w, num, den
[Gre, Gim, w] = addp(Gre,Gim,w, num, den);
range = region(Grootloc);
limit = wsize(Gre,Gim);
tpt1 = testpt(range,limit);
cnum = clnum(num,den, tpt1);
[spos,n1] = stab1(cnum);
kregion1 = kreg(spos,range);
if nargin == 3, 
   kmin = c;
elseif nargin == 4,
   kmin = c; kmax = d;
end 
% newk indicates if NaN is used in the 3 or 5 arguments
% (NaN, kmax) is treated as a kmin problem 
if nargin == 3 | nargin == 5,
   s = chkcir(range,spos,kmin); 
   newk = 0;  
elseif nargin == 4,
   if isnan(c), 
      s = chkcir(range,spos,-d);
      kmin = -d; 
      newk = 1;
   else
      s = chkcir(range,spos,kmin,kmax);
      newk=0;
   end 
elseif nargin == 6,
   if isnan(kmin), 
      s = chkcir(range,spos,-kmax);
      kmin = -kmax;
      newk = 1;
   else
      s = chkcir(range,spos,kmin,kmax);
      newk = 0;
   end 
end

% unstable at kmin=0 and a plant that starts at -inf or +inf 
if  kmin == 0 & fdmin(num,den,0) == inf, s = 0; end
if  kmin == 0 & fdmin(num,den,0) == -inf, s = 0; end
if s,   
   [num,den] = addz(num,den);
   % here den1 is the denominator of the closed loop system
   den1 = den + kmin*num; 
   G_tilde = fdmin(num,den1,kmin);
   % special case when G_tilde become zero, Gmax1=+inf   
   if G_tilde == 0,
      kmax1 = inf; 
      Gmax1 = 0;
   else  
      kmax1 = -1/G_tilde + kmin;
      Gmax1 = -1/kmax1;
   end  
   if nargin == 3 | nargin == 5 | newk == 1,                  
      disp('circle criterion is satisfied');
      if newk, 
         disp(['minimum sector bound F_min = ', num2str(-kmax1)]);
         plotc(-kmax1, -kmin,num,-den, limit);
         xlabel('Real Axis');
         ylabel('Imaginary Axis');
         title('Nyquist and Circle Criterion Diagram');
      else
         disp(['maximum sector bound F_max = ', num2str(kmax1)]);  
         plotc(kmin, kmax1,num,den,limit);
         xlabel('Real Axis');
         ylabel('Imaginary Axis');
         title('Nyquist and Circle Criterion Diagram');
      end
   else 
      if kmax < kmax1,   
         disp('circle criterion is satisfied');  
      else
         disp('circle criterion is not satisfied');  
      end        
      plotc(kmin,kmax, num, den, limit);   
      xlabel('Real Axis');
      ylabel('Imaginary Axis');
      title('Nyquist and Circle Criterion Diagram');
  end            
else
   disp('circle criterion is not satisfied');     
end
% % take care of the output if requested
if (mynargout == 2) & (nargin == 3 | nargin == 5)
    Fmin = kmin; 
    Fmax = kmax1;
elseif (mynargout == 2) & (nargin == 4 | nargin == 6)
    Fmin = -kmax1;
    Fmax = -kmin;
end
