function [kregion] = kreg(spos, range)

% function [kregion] = kreg(spos, range)
% determines the range of stable k values. 
% KREG returns stable k range in two columns with lower and upper bound
% SPOS is the stability position (see STAB1 m-file function)
% RANGE is the "Gre"(each point on the real axis of G(s) corresponds
% to k = -1/Gre. (see REGION m-file function for more details).

% So, KREG checks the region on the real axis that gives the closed loop
% stability and converts the bounds of the region into its corresponding
% gain k range.

% There are alltogether 11 cases where the region can lie : 

%   3 cases exist when one bound is either +inf or -inf .
%   2 cases exist for one bound that is zero.
%   3 more cases exist for both bounds that are either all positives,
%    all negatives or is between.
% If the stable region occurs at both +/- infinity, the 2 gain k ranges 
% are combined.

% 96 May 27,28,29,30 June 3,5,7,10,17, Sept 22, 
% 97 May 6,7
% 2004 June 30  July 1

if nargin == 0
    display('USAGE: [kregion]= kreg(spos,range)')
    return
elseif nargin < 2
    error('check the number input arguments you mentioned!')
end 
k = 1;   % counter
% initialize kregion to empty  
kregion = [];
for x = 1:length(spos)
  if spos(x) > 0     % stable system
      
      % 3 cases for one bound is -inf
     if range(x,1) == -inf 
        if range(x,2) < 0,  
           kregion(k,1) = 0; kregion(k,2) = -1/range(x,2);
           k = k + 1;
        elseif range(x,2) == 0
           kregion(k,1) = 0; kregion(k,2) = inf;   % kregion(x,2) = -1/0 = inf
           k = k + 1;
        else 
           kregion(k,1) = 0; kregion(k,2) = inf;
           kregion(k+1,1) = -inf; kregion(k+1,2) = -1/range(x,2); 
           k = k + 2;
        end
        
      % 3 cases for one bound is inf
     elseif range(x,2) == inf,
        if range(x,1) > 0, 
           kregion(k,1) = -1/range(x,1); kregion(k,2) = 0;
           k = k + 1;
        elseif range(x,1) == 0,
           kregion(k,1) = -inf; kregion(k,2) = 0;
           k = k + 1;
        else
           kregion(k,1) = -1/range(x,1); kregion(k,2) = inf;
           kregion(k+1,1) = -inf; kregion(k+1,2) = 0;
           k = k + 2;
        end
        
     % 2 cases for one bound is zero
     elseif range(x,2) == 0;
        kregion(k,1) = -1/range(x,1); kregion(k,2) = inf;
        k = k + 1;
     elseif range(x,1) == 0;
        kregion(k,1) = -inf; kregion(k,2) = -1/range(x,2);
        k = k + 1; 
        
    % 3 cases for both bounds are either + or - or in between
     elseif range(x,2) < 0,
        kregion(k,1) = -1/range(x,1); kregion(k,2) = -1/range(x,2);
        k = k + 1;
     elseif range(x,1) > 0,
        kregion(k,1) = -1/range(x,1); kregion(k,2) = -1/range(x,2);
        k = k + 1;
     else
        kregion(k,1) = -1/range(x,1); kregion(k,2) = inf;
        kregion(k+1,1) = -inf; kregion(k+1,2) = -1/range(x,2);
        k = k + 2;
     end
   end    
end
% combine two ranges at -inf & +inf if possible
if ~isempty(kregion),
   if kregion(1,1) == 0 & kregion(size((kregion),1),2) == 0,
      kregion(size(kregion,1),2) = kregion(1,2);
      kregion(1,:) = [];
   end
end