function [Fmin,Fmax] = popov(a,b,c,d,kmin,kmax)

% POPOV   An implementation of the Popov Criterion (PC) for a non-
%         linear feedback control system with a linear part G(s)
%         and a time-invariant nonlinearity f(y) lying in a sector
%         [ F_min, F_max ], i.e.,
%                        F_min < f(y)/y < F_max
%         If F_min is not 0, G(s) and f(y) are transformed to obtain
%         the special case [ 0, (F_max - F_min) ] by defining g(y) =
%         f(y) - F_min*y, G(s) = G(s)/(1 + F_min*G(s)).  The special
%         case is uniformly asymptotically stable in the large (UASIL)
%         if a line can be drawn through the point -1/(F_max - F_min)
%         such that the Popov locus of G(jw) lies to the right; the
%         Popov locus is the plot of w*Im(G(jw)) vs Re(G(jw)).
%
%         POPOV(A,B,C,D,F_min,F_max) will check if the PC is satisfied
%         for the sector-bounded nonlinearity and G(s) = C(sI-A) B + D
%
%         POPOV(A,B,C,D,F_min) will determine the maximum value of
%         F_max for which the PC is satisfied.
%
%         POPOV(A,B,C,D,NaN,F_max) will determine the minimum value of
%         F_min for which the PC is satisfied.
%
%         POPOV(NUM,DEN,F_min,F_max) et cetera are corresponding forms
%         for G(s) provided in ratio of polynomial form.
%
%         [FMIN,FMAX]=POPOV(A,B,C,D,F_min) et cetera will return the
%         sector bounds for which the PC is satisfied (even if, for
%         example, F_min,F_max are supplied for which the PC is NOT OK).
%
%         C. Chan and J.H. Taylor, Univ. of New Brunswick, July. 1997.
%
%         Reference: Narendra & Taylor "Frequency Domain Criteria for
%         Absolute Criteria", Academic Press, 1973.

% 97 Jan 17,18,20,27,28,29,30,31,Feb 3,10,11,17,18,21,22,23,24,26
% 97 July - patched to work (addpold etc.) WITHOUT trying to
%           transform the line (which can't be done) - JH Taylor
% 99 Apr 30 - updated to accommodate new NaN syntax requirements

global REnum REden IMnumPop reT 

if nargin==3 | nargin==4,
   num=a; den=b;      
   [Grootloc,freq] = findrt(num,den);
elseif nargin==5 | nargin==6,
   [Grootloc,freq,num,den]=findrt(a,b,c,d);
else
   error('wrong  number of input arguments');
end 
[Gre,Gim,w]=nyq4p2(num,den);    
[Grootloc,freq]=addr(num,den,Grootloc,freq);
[Gre,Gim,w]=addp(Gre,Gim,w, num, den);
range = region(Grootloc);
limit=wsize(Gre,Gim);
tpt1 = testpt(range,limit);
cnum=clnum(num,den,tpt1);
[spos,n1] = stab1(cnum);
kregion1=kreg(spos,range);

if nargin==3, 
   kmin=c;
elseif nargin==4,
   kmin=c; kmax=d;
end 

if nargin==3 | nargin==5,
   if isnan(kmin), error('single bound cannot be NaN'); end
end

if nargin==3 | nargin==5,
   s=chkcir(range,spos,kmin);
else
   s=chkcir(range,spos,kmin,kmax);
end

% unstable at kmin=0 and a plant that starts at -inf or +inf 
if  kmin==0 & fdmin(num,den,0)==inf, s=0; end
if  kmin==0 & fdmin(num,den,0)==-inf, s=0; end

if s,

   %perform appropriate transformation if kmin is not zero
   if isnan(kmin),
      [num,den]=addz(num,den);
      den=den+kmax*num; 
      num=-num;
   elseif kmin~=0,
      [num,den]=addz(num,den);
      den=den+kmin*num; 
   end  
   
   [REnum,REden,IMnum,IMden]=g2reim(num,den);
   % get the popov function by multiplying b with w 
   IMnumPop=[IMnum,0];
   % find the slope at the left most crossing of the Popov curve
   % obtain the derivative function of real and imaginary part
   [re,slope,root]=fdlin(REnum,REden,IMnumPop,IMnum);
   [REnum,IMnumPop]=addz(REnum,IMnumPop);
   iscase1=pcase1(REnum,REden,IMnumPop,re,slope,root);    
   if ~iscase1,
      ang=guess(re,REnum,REden,IMnumPop,IMden);
      rad=fzero4p2('angfun2',ang);
      % reT is a global variable found in angfun2(rad) 
      reT1=min(reT);
      re=reT1/cos(rad);
      slope=tan(pi/2-rad);
   end

   % does Popov line pass through the origin?
   if abs(re)<eps,
      kmax0=inf*sign(re);
   else    
      kmax0=-1/re;
   end

   % set up decision tree (it's complicated!)
   if nargin==3 | nargin==5,
      %only kmin given; do we have a meaningful kmax?
      if kmax0 > 0,
         disp('Popov criterion is satisfied'); 
         kmax=kmin+kmax0;
         disp(['maximum sector bound F_max = ',num2str(kmax)]);  
      else    
         disp(['Popov criterion cannot be satisfied for F_min = ',num2str(kmin)]);
      end
   else
      %_either_ kmin, kmax given for OK _or_ kmax given kmin desired
      if isnan(kmin),
         %kmax given kmin desired
         if kmax0 > 0,
            disp('Popov criterion is satisfied'); 
            kmin=kmax-kmax0;
            disp(['minimum sector bound F_min = ',num2str(kmin)]);  
         else    
            disp(['Popov criterion cannot be satisfied for F_max = ',num2str(kmax)]);
         end
      else
         %both kmin and kmax given
         if kmax0 > 0,
            disp('Popov criterion is satisfied for your F_min, F_max'); 
         else
            disp('Popov criterion is not satisfied');    
         end
      end
   end 
   [xT,yT,gx,gy]=plotln(num,den,REnum,REden,IMnumPop,re,slope);
   title('Popov criterion','fontsize',11);
   xlabel('Real Axis: Re(G(j\omega))');
   ylabel('Imaginary Axis: \omega \times Im(G(j\omega))')
else
   disp('Popov criterion is not satisfied');     
end
% take care of outputs if requested:
if nargout~=0,
   if s == 0,
     Fmin = []; Fmax = [];
   else
     Fmin = kmin; Fmax = kmax;
   end
end
